
#########################################################################
#									
# 	This R Program computes recommended regimen 
#	for the method of Wages (2018)* 											
#	*Wages, N. Flexible Phase I-II design for partially ordered regimens 
#	sumbitted to Pharm Stat
#									
#########################################################################
#install.packages("dfcrm")
#install.packages("nnet")
library(dfcrm)
library(nnet)

regimen.rec<-function(p.skel,q.skel,tul,ell,n.ar,n.stop,curr.comb,cs,cf,y,z,n){


    ncomb=length(n)
    ptox.hat = numeric(ncomb); # estimate of toxicity prob
    peff.hat = numeric(ncomb); # estimate of efficacy prob
    p.safety=numeric(ncomb)
    p.futility=numeric(ncomb)



# if a single ordering is inputed as a vector, convert it to a matrix
	if(is.vector(p.skel)) p.skel=t(as.matrix(p.skel));
	
	nord.tox = nrow(p.skel);
	mprior.tox = rep(1/nord.tox, nord.tox);  # prior for each toxicity ordering

# if a single ordering is inputed as a vector, convert it to a matrix
	if(is.vector(q.skel)) q.skel=t(as.matrix(q.skel));
	
	nord.eff = nrow(q.skel);
	mprior.eff = rep(1/nord.eff, nord.eff);#c(0.34,rep(0.02,33))#rep(1/nord.eff, nord.eff);  # prior for each efficacy ordering

post.tox<-function(a,p,y,n){
	s2=0.875^2
	lik=1
	for(j in 1:length(p)){
		pj=p[j]**exp(a)
		lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
		}
	return(lik*exp(-0.5*a*a/s2));
    }

#the posterior mean of ptox
    posttoxf <- function(a, p, y, n, j) { p[j]^(exp(a))*post.tox(a, p, y, n); }
	
post.eff<-function(b,q,z,n){
	s2=1
	lik=1
	for(j in 1:length(q)){
		qj=q[j]**exp(b)
		lik=lik*qj^z[j]*(1-qj)^(n[j]-z[j]);
		}
	return(lik*exp(-0.5*b*b/s2));
    }

#the posterior mean of peff
    postefff <- function(b, q, z, n, j) {q[j]^(exp(b))*post.eff(b, q, z, n); }


	

		marginal.tox = rep(0, nord.tox);
		for(k in 1:nord.tox)
		{
			marginal.tox[k] = integrate(post.tox,lower=-Inf,upper=Inf, p=p.skel[k,], y=y,n=n)$value;
		}
		
		postprob.tox = (marginal.tox*mprior.tox)/sum(marginal.tox*mprior.tox);

		marginal.eff = rep(0, nord.eff);
		for(k in 1:nord.eff)
		{
			marginal.eff[k] = integrate(post.eff,lower=-Inf,upper=Inf, q=q.skel[k,], z=z, n=n)$value;
		}
		
		postprob.eff = (marginal.eff*mprior.eff)/sum(marginal.eff*mprior.eff);
		
		# toxicity model selection, identify the model with the highest posterior prob
			if(nord.tox>1){ 
				mtox.sel = which.max(postprob.tox); 
			} else{
				mtox.sel = 1;
			}

		# efficacy model selection, identify the model with the highest posterior prob
			if(nord.eff>1){
				meff.sel = which.is.max(postprob.eff); 
			} else{
				meff.sel = 1;
			}

		# calculate posterior mean of toxicity probability at each combo
			for(j in 1:ncomb){
				ptox.hat[j] = integrate(posttoxf,lower=-Inf,upper=Inf, p.skel[mtox.sel,], y, n,j)$value/marginal.tox[mtox.sel]; 
				p.safety[j] = integrate(post.tox,lower=log(log(tul)/log(p.skel[mtox.sel,j]+0.01)),upper=Inf, p.skel[mtox.sel,], y, n)$value/marginal.tox[mtox.sel];
			}
		

		# calculate posterior mean of efficacy probability at each combo
			for(j in 1:ncomb){
				peff.hat[j] = integrate(postefff,lower=-Inf,upper=Inf, q.skel[meff.sel,], z, n,j)$value/marginal.eff[meff.sel]; 
				p.futility[j] = integrate(post.eff,lower=-Inf,upper=log(log(ell)/log(q.skel[meff.sel,j]+0.01)),q.skel[meff.sel,], z, n)$value/marginal.eff[meff.sel];
				}

		
		if(p.safety[which.min(ptox.hat)]<cs){
				stops<-1
				break
				}




		aset=which(p.safety>cs & p.futility>cf)
		if(length(aset)==0){
			stop=1
			break
		}

		utility=rep(0,ncomb)
		utility[aset]=peff.hat[aset]*(1-ptox.hat[aset])
		ri=utility/sum(utility)
		best=ifelse(sum(n)<n.ar,sample(1:ncomb,1,prob=ri),which.max(utility))
		if(sum(n)==1){
			comb.next<-2
		} else if(sum(n)==2){
			comb.next<-3
		} else{
			comb.next<-best
		}	

		
 stop<-NULL
 ifelse(n[comb.next]>=n.stop,stop<-"YES, STOP STUDY",stop<-"NO, CONTINUE STUDY")
 out<-list(Sys.time(),y,z,n,round(ptox.hat,3),aset,round(utility,3),which.max(utility),comb.next,stop)
 names(out)<-c('Date and Time','Observed DLTs','Observed Responses','Number of patients treated','Estimated DLT probabilities','Safe regimens','Estimated success probabilities','Safe regimen with maximum desirability','Regimen recommendation','Stopping rule triggered?')
 print(out)
}




###############Input
d<-6

#####number of parallel working models
s<-6 ##for toxicity
g<-6 ##for efficacy

###Specify the possible DLT orderings of the regimens
orders<-matrix(nrow=s,ncol=d)
orders[1,]<-c(1,2,3,4,5,6)	#by rows
orders[2,]<-c(1,2,3,4,6,5)	#by columns
orders[3,]<-c(2,1,3,5,4,6)	#up diagonals
orders[4,]<-c(2,3,1,5,6,4)	#alternate down-up diagonals
orders[5,]<-c(3,1,2,6,4,5)	#up diagonals
orders[6,]<-c(3,2,1,6,5,4)	#alternate down-up diagonals

###Specify skeleton for DLT probabilities
skeleton<-round(getprior(0.04,0.30,1,6),2)

###The following will adjust the location of each skeleton value to correspond 
###to the 's' possible orderings specified above. 
###'alpha' is the matrix of skeleton values
p.skel<-matrix(0,nrow=s,ncol=d)
for(j in 1:s){ 
	p.skel[j,]<-skeleton[order(orders[j,])]
}

###Specify the possible efficacy orderings of the regimens
###The following will adjust the location of each skeleton value to correspond 
###to the 'g' possible orderings specified above. 
###'beta' is the matrix of skeleton values
q.skel<-matrix(0,nrow=g,ncol=d)
q.skel[1,]<-c(0.30,0.20,0.20,0.50,0.40,0.40)+0.20
q.skel[2,]<-c(0.40,0.30,0.30,0.40,0.30,0.30)+0.20
q.skel[3,]<-c(0.20,0.30,0.20,0.40,0.50,0.40)+0.20
q.skel[4,]<-c(0.30,0.40,0.30,0.30,0.40,0.30)+0.20
q.skel[5,]<-c(0.20,0.20,0.30,0.40,0.40,0.50)+0.20
q.skel[6,]<-c(0.30,0.30,0.40,0.30,0.30,0.40)+0.20



tul<-0.30     ##toxicity upper limit 
ell<-0.20     ##efficacy lower limit
start.comb=1  ##starting combination
cs=cf=0.10	  ##thresholds used to define stopping rules
n.stop=30	  ##number needed on one combo to stp
n.ar=70/3	  ##size of the adaptive randomization stage

y=c(0,0,0,0,0,1)  ##number of DLTs on each regimen
z=c(0,0,1,0,1,1)	##number of responses on each regimen
n=c(1,1,1,2,1,2)	##number of patients treated on each regimen
comb.curr=4  	##current regimen
regimen.rec(p.skel,q.skel,tul,ell,n.ar,n.stop,curr.comb,cs,cf,y,z,n)



