
#########################################################################
#									
# 	This R Program simulates the design of Wages (2018)* 				
#									
#	*Wages, NA. (2018) Flexible Phase I/II design 
#	for partially ordered regimens.	submitted to Pharm Stat
#									
#########################################################################



###install and load required R packages
#install.package("nnet")
#install.packages("dfcrm")
library(nnet)
library(dfcrm)


###Load the function 'bpocrm' 
bpocrm<-function(p0,q0,p.skel,q.skel,tul,ell,cohortsize,n.ar,ncohort,n.stop,start.comb,cs,cf){
 
# if a single ordering is inputed as a vector, convert it to a matrix
	if(is.vector(p.skel)) p.skel=t(as.matrix(p.skel));
	
	nord.tox = nrow(p.skel);
	mprior.tox = rep(1/nord.tox, nord.tox);  # prior for each toxicity ordering

# if a single ordering is inputed as a vector, convert it to a matrix
	if(is.vector(q.skel)) q.skel=t(as.matrix(q.skel));
	
	nord.eff = nrow(q.skel);
	mprior.eff = rep(1/nord.eff, nord.eff);#c(0.34,rep(0.02,33))#rep(1/nord.eff, nord.eff);  # prior for each efficacy ordering

post.tox<-function(a,p,y,n){
	s2=0.875^2
	lik=1
	for(j in 1:length(p)){
		pj=p[j]**exp(a)
		lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
		}
	return(lik*exp(-0.5*a*a/s2));
    }

#the posterior mean of ptox
    posttoxf <- function(a, p, y, n, j) { p[j]^(exp(a))*post.tox(a, p, y, n); }
	
post.eff<-function(b,q,z,n){
	s2=1
	lik=1
	for(j in 1:length(q)){
		qj=q[j]**exp(b)
		lik=lik*qj^z[j]*(1-qj)^(n[j]-z[j]);
		}
	return(lik*exp(-0.5*b*b/s2));
    }

#the posterior mean of peff
    postefff <- function(b, q, z, n, j) {q[j]^(exp(b))*post.eff(b, q, z, n); }



### run a trial 	
    ncomb = ncol(p.skel);   #number of combos
    y=rep(0,ncomb);  #number of toxicity/responses at each dose level
    z=rep(0,ncomb);   #number of efficacy at each dose level
    n=rep(0,ncomb);  #number of treated patients at each dose level
    comb.curr = start.comb;  # current dose level	 
    ptox.hat = numeric(ncomb); # estimate of toxicity prob
    peff.hat = numeric(ncomb); # estimate of efficacy prob
    p.safety=numeric(ncomb)
    p.futility=numeric(ncomb)
    comb.select=rep(0,ncomb); # a vector of indicators for dose selection
    stop=stops=stopf=0; #indicate if trial stops early
    i=1	
while(i <= ncohort)
    {
	# generate data for a new cohort of patients
		lasttox=rbinom(1,cohortsize,p0[comb.curr]);
		y[comb.curr] = y[comb.curr] + lasttox;
		z[comb.curr] = z[comb.curr] + rbinom(1,cohortsize,q0[comb.curr]);
		n[comb.curr] = n[comb.curr] + cohortsize;		

		if(any(n>n.stop)){
			stop<-0
			break
		}

		marginal.tox = rep(0, nord.tox);
		for(k in 1:nord.tox)
		{
			marginal.tox[k] = integrate(post.tox,lower=-Inf,upper=Inf, p=p.skel[k,], y=y,n=n)$value;
		}
		
		postprob.tox = (marginal.tox*mprior.tox)/sum(marginal.tox*mprior.tox);

		marginal.eff = rep(0, nord.eff);
		for(k in 1:nord.eff)
		{
			marginal.eff[k] = integrate(post.eff,lower=-Inf,upper=Inf, q=q.skel[k,], z=z, n=n)$value;
		}
		
		postprob.eff = (marginal.eff*mprior.eff)/sum(marginal.eff*mprior.eff);
		
		# toxicity model selection, identify the model with the highest posterior prob
			if(nord.tox>1){ 
				mtox.sel = which.max(postprob.tox); 
			} else{
				mtox.sel = 1;
			}

		# efficacy model selection, identify the model with the highest posterior prob
			if(nord.eff>1){
				meff.sel = which.is.max(postprob.eff); 
			} else{
				meff.sel = 1;
			}

		# calculate posterior mean of toxicity probability at each combo
			for(j in 1:ncomb){
				ptox.hat[j] = integrate(posttoxf,lower=-Inf,upper=Inf, p.skel[mtox.sel,], y, n,j)$value/marginal.tox[mtox.sel]; 
				p.safety[j] = integrate(post.tox,lower=log(log(tul)/log(p.skel[mtox.sel,j]+0.01)),upper=Inf, p.skel[mtox.sel,], y, n)$value/marginal.tox[mtox.sel];
			}
		

		# calculate posterior mean of efficacy probability at each combo
			for(j in 1:ncomb){
				peff.hat[j] = integrate(postefff,lower=-Inf,upper=Inf, q.skel[meff.sel,], z, n,j)$value/marginal.eff[meff.sel]; 
				p.futility[j] = integrate(post.eff,lower=-Inf,upper=log(log(ell)/log(q.skel[meff.sel,j]+0.01)),q.skel[meff.sel,], z, n)$value/marginal.eff[meff.sel];
				}

		
	aset=which(p.safety>cs & p.futility>cf)
		if(length(aset)==0){
			stop=1
			break
		}

		utility=rep(0,ncomb)
		utility[aset]=peff.hat[aset]*(1-ptox.hat[aset])
		ri=utility/sum(utility)
		best=ifelse(sum(n)<n.ar,sample(1:ncomb,1,prob=ri),which.max(utility))
		if(i==1){
			comb.curr<-2
		} else if(i==2){
			comb.curr<-3
		} else{
			comb.curr<-best
		}	
 
		i=i+1
 		}

	if(stop==0 & stops==0 & stopf==0){
		comb.select[comb.curr]=comb.select[comb.curr]+1;
		}
	return(list(comb.select=comb.select,tox.data=y,eff.data=z,pt.allocation=n,stopf=stopf,stops=stops))
}
##########'bpocrm' end here

###Load the function 'bpocrm.sim' 
bpocrm.sim<-function(p0,q0,p.skel,q.skel,tul,ell,cohortsize,n.ar,ncohort,n.stop,ntrial,start.comb,cs,cf){
	ncomb=length(p0)
	
	comb.select<-y<-z<-n<-matrix(nrow=ntrial,ncol=ncomb)
	nstopf=nstops=0
	
	for(i in 1:ntrial){
		result<-bpocrm(p0,q0,p.skel,q.skel,tul,ell,cohortsize,n.ar,ncohort,n.stop,start.comb,cs,cf)
		comb.select[i,]=result$comb.select
		y[i,]=result$tox.data
		z[i,]=result$eff.data
		n[i,]=result$pt.allocation
		nstopf=nstopf+result$stopf
		nstops=nstops+result$stops
	}

	cat("True toxicity probability:\n")
	cat(round(p0,3),  sep="\t",  "\n")
	cat("True activity probability:\n")
	cat(round(q0,3), sep="\t",  "\n")
	cat("Selection percentage:\n")
	cat(formatC(colMeans(comb.select)*100, digits=1, format="f"), sep="\t",  "\n")
	cat("Average number of toxicities:\n")
	cat(formatC(colMeans(y), digits=1, format="f"), sep="\t",   "\n")
	cat("Average number of responses:\n")
	cat(formatC(colMeans(z), digits=1, format="f"), sep="\t",   "\n")
	cat("Average number of patients treated:\n")
	cat(formatC(colMeans(n), digits=1, format="f"), sep="\t",   "\n")
	cat("Percentage of trials stopped for safety:\n")
	cat(nstops/ntrial*100, "\n")
	cat("Percentage of trials stopped for futility:\n")
	cat(nstopf/ntrial*100, "\n")
}
##########'bpocrm.sim' end here




###############Simualtions
d<-6

#####number of possible orderings
s<-6 ##for toxicity
g<-6 ##for efficacy

###Specify the possible DLT orderings of the drug combinations
orders<-matrix(nrow=s,ncol=d)
orders[1,]<-c(1,2,3,4,5,6)	#by rows
orders[2,]<-c(1,2,3,4,6,5)	#by columns
orders[3,]<-c(2,1,3,5,4,6)	#up diagonals
orders[4,]<-c(2,3,1,5,6,4)	#alternate down-up diagonals
orders[5,]<-c(3,1,2,6,4,5)	#up diagonals
orders[6,]<-c(3,2,1,6,5,4)	#alternate down-up diagonals

###Specify skeleton for DLT probabilities
skeleton<-round(getprior(0.04,0.30,1,6),2)

###The following will adjust the location of each skeleton value to correspond 
###to the 's' possible orderings specified above. 
###'alpha' is the matrix of skeleton values
p.skel<-matrix(0,nrow=s,ncol=d)
for(j in 1:s){ 
	p.skel[j,]<-skeleton[order(orders[j,])]
}

###Specify the possible efficacy orderings of the drug combinations
###The following will adjust the location of each skeleton value to correspond 
###to the 'g' possible orderings specified above. 
###'beta' is the matrix of skeleton values
q.skel<-matrix(0,nrow=g,ncol=d)
q.skel[1,]<-c(0.30,0.20,0.20,0.50,0.40,0.40)+0.20
q.skel[2,]<-c(0.40,0.30,0.30,0.40,0.30,0.30)+0.20
q.skel[3,]<-c(0.20,0.30,0.20,0.40,0.50,0.40)+0.20
q.skel[4,]<-c(0.30,0.40,0.30,0.30,0.40,0.30)+0.20
q.skel[5,]<-c(0.20,0.20,0.30,0.40,0.40,0.50)+0.20
q.skel[6,]<-c(0.30,0.30,0.40,0.30,0.30,0.40)+0.20

###Toxicity scenarios in Tables 3 and 4
p1<-c(0.05,0.01,0.07,0.08,0.17,0.18)
p2<-c(0.09,0.05,0.20,0.44,0.25,0.50)
p3<-c(0.15,0.10,0.40,0.35,0.45,0.60)

###Activity scenarios in Tables 3 and 4
q1<-c(0.19,0.15,0.30,0.50,0.40,0.70)
q2<-c(0.35,0.40,0.60,0.35,0.40,0.60)
q3<-c(0.40,0.15,0.30,0.60,0.35,0.40)  
q4<-c(0.50,0.35,0.30,0.50,0.35,0.30)
q5<-c(0.30,0.65,0.40,0.45,0.75,0.55)
q6<-c(0.55,0.75,0.50,0.55,0.75,0.50)

###Supplemental scenarios
#Toxicity 
p1s<-c(0.05,0.01,0.07,0.08,0.17,0.18)
p2s<-c(0.05,0.12,0.20,0.50,0.40,0.55)

#Activity
q1s<-c(0.50,0.40,0.70,0.19,0.15,0.30)
q2s<-c(0.60,0.35,0.40,0.40,0.15,0.30)

##Design specifications
tul<-0.30     ##toxicity upper limit 
ell<-0.20     ##efficacy lower limit
cohortsize=1  ##cohort size for each inclusion
ncohort=70    ##number of cohorts
start.comb=1  ##starting combination
n.stop=30	  ##number needed on one combo to stop
ntrial=10   ##number of simulated trials 
cs=cf=0.05	  ##thresholds used to define stopping rules
n.ar=70/3	  ##size of adaptive randomization phase

##True probability scenario
p0<-p1 #true toxicity probability scenario
q0<-q1 #true efficacy probability scenario

##simulate a single trial
set.seed(580) ##random seed
bpocrm(p0,q0,p.skel,q.skel,tul,ell,cohortsize,n.ar,ncohort,n.stop,start.comb,cs,cf)

##simulate multiple trials
set.seed(580) ##random seed
bpocrm.sim(p0,q0,p.skel,q.skel,tul,ell,cohortsize,n.ar,ncohort,n.stop,ntrial,start.comb,cs,cf)




